//
//  TokenIcon.swift
//  FreeOTP
//
//  Created by Justin Stephenson on 2/12/20.
//  Copyright © 2020 Fedora Project. All rights reserved.
//

import Foundation
import FontAwesome

public extension UIColor {
    convenience init(hexString: String, alpha: CGFloat = 1.0) {
        let hexString: String = hexString.trimmingCharacters(in: CharacterSet.whitespacesAndNewlines)
        let scanner = Scanner(string: hexString)

        if (hexString.hasPrefix("#")) {
            scanner.scanLocation = 1
        }

        var color: UInt32 = 0
        scanner.scanHexInt32(&color)

        let mask = 0x000000FF
        let r = Int(color >> 16) & mask
        let g = Int(color >> 8) & mask
        let b = Int(color) & mask

        let red   = CGFloat(r) / 255.0
        let green = CGFloat(g) / 255.0
        let blue  = CGFloat(b) / 255.0

        self.init(red:red, green:green, blue:blue, alpha:alpha)
    }
}

extension UIImage {
    func addImagePadding(x: CGFloat, y: CGFloat) -> UIImage? {
        let width: CGFloat = size.width + x
        let height: CGFloat = size.height + y
        UIGraphicsBeginImageContextWithOptions(CGSize(width: width, height: height), false, 0)
        let origin: CGPoint = CGPoint(x: (width - size.width) / 2, y: (height - size.height) / 2)
        draw(at: origin)
        let imageWithPadding = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()

        return imageWithPadding
    }
}

public struct IconData {
    public var iconImg: UIImage! = nil
    public var bgColor: String = ""
    public var name: String = ""
}

public struct IconMatch {
    var name: String
    var levDist: Int
}

public enum faIconType {
    case brands
    case solid
    case recommended
}

public class TokenIcon {
    let defaults = UserDefaults.standard

    var iconsBrand = [(name: String, value: String)]()
    var iconsSolid = [(name: String, value: String)]()

    public init() {
        for (key, value) in faIconsBrands.sorted(by: <) {
            iconsBrand += [(name: "\(key)", value: "\(value)")]
        }
        for (key, value) in faIconsSolid.sorted(by: <) {
            iconsSolid += [(name: "\(key)", value: "\(value)")]
        }
    }

    public func saveMapping(issuer: String, iconName: String, iconColor: String) {
        let iconDict = ["Name": iconName, "Color": iconColor]
        defaults.set(iconDict, forKey: issuer)
    }

    public func loadMapping(issuer: String) -> [String: String]? {
        guard let iconName = defaults.object(forKey: issuer) as? [String: String] else {
            return nil
        }
        return iconName
    }

    public func getfaIconName(for brand: String) -> String! {
        let brand = brand.lowercased().replacingOccurrences(of: " ", with: "")

        if !brand.isEmpty {
            for icons in iconsBrand {
                if icons.name == brand.lowercased() {
                    return icons.value
                }
            }
        }
        return nil
    }

    public func getBrandColor(_ key: String) -> UIColor! {
        let brand = key.lowercased().replacingOccurrences(of: " ", with: "")
        if let val = iconBGColorMapping[brand] {
            return UIColor(hexString: val)
        }

        return nil
    }

    public func getBrandColorHex(_ key: String) -> String! {
        let brand = key.lowercased().replacingOccurrences(of: " ", with: "")
        if let val = iconBGColorMapping[brand] {
            return val
        }

        return nil
    }

    public func getBackgroundColor(name: String, uriColor: String = "") -> UIColor {
        // Use URI-provided color
        if uriColor != "" {
            return UIColor(hexString: uriColor)
        }
        // Use color from brand dictionary
        if let color = getBrandColor(name) {
            return color
        }
        // Fallback to predictable hash-based color
        return name == "" ? UIColor.systemGray : getBackgroundColorHash(name)
    }

    func sha256(data : Data) -> Data {
        var hash = [UInt8](repeating: 0,  count: Int(CC_SHA256_DIGEST_LENGTH))
        data.withUnsafeBytes {
            _ = CC_SHA256($0.baseAddress, CC_LONG(data.count), &hash)
        }
        return Data(hash)
    }

    public func getBackgroundColorHash(_ issuer: String) -> UIColor {
        let mydata = sha256(data: Data(issuer.utf8))
        let number = mydata.withUnsafeBytes( {
            $0.load(as: Int.self )
        })
        let idx = abs(number)
        let index = idx % backgroundColors.count
        return UIColor(hexString: backgroundColors[index])
    }

    public func getFontAwesomeIcon(issuer: String, _ size: CGSize = CGSize(width: 30, height: 30)) -> UIImage! {
        let faIconName = "fa-" + issuer.lowercased().replacingOccurrences(of: " ", with: "")
        if let iconName = FontAwesome(rawValue: faIconName) {
            return UIImage.fontAwesomeIcon(name: iconName, style: .brands, textColor: .white, size: size)
        }

        return nil
    }

    public func getFontAwesomeIcon(faName: String, faType: faIconType, size: CGSize = CGSize(width: 30, height: 30)) -> UIImage! {
        switch faType {
        case .brands:
            if let iconName = FontAwesome(rawValue: faName) {
                return UIImage.fontAwesomeIcon(name: iconName, style: .brands, textColor: .white, size: size)
            }
        case .solid:
            if let iconName = FontAwesome(rawValue: faName) {
                return UIImage.fontAwesomeIcon(name: iconName, style: .solid, textColor: .white, size: size)
            }
        case .recommended:
            if let iconName = FontAwesome(rawValue: faName) {
                return UIImage.fontAwesomeIcon(name: iconName, style: .brands, textColor: .white, size: size)
            }
        }

        return nil
    }

    public func getCustomIcon(issuer: String, size: CGSize =  CGSize(width: 30, height: 30)) -> IconData! {
        var icon = IconData()

        if let faName = issuerBrandMapping[issuer] {
            if let faIcon = FontAwesome(rawValue: faName) {
                icon.iconImg = UIImage.fontAwesomeIcon(name: faIcon, style: .brands, textColor: .white, size: size)
                icon.name = faName
                let color = iconBGColorMapping[faName]
                icon.bgColor = color ?? "#6F7171"
                
                return icon
            }
        }

        if issuer.isEmpty {
            return nil
        }

        if let iconMapping = loadMapping(issuer: issuer) {
            if faIconsBrands.values.contains(iconMapping["Name"]!) {
                if let iconName = FontAwesome(rawValue: iconMapping["Name"]!) {
                    icon.iconImg = UIImage.fontAwesomeIcon(name: iconName, style: .brands, textColor: .white, size: size)
                    icon.bgColor = iconMapping["Color"] ?? "#6F7171"
                    icon.name = iconMapping["Name"]!

                    return icon
                }
            }

            if faIconsSolid.values.contains(iconMapping["Name"]!) {
                if let iconName = FontAwesome(rawValue: iconMapping["Name"]!) {
                    icon.iconImg = UIImage.fontAwesomeIcon(name: iconName, style: .solid, textColor: .white, size: size)
                    icon.bgColor = iconMapping["Color"] ?? "#6F7171"
                    icon.name = iconMapping["Name"]!

                    return icon
                }
            }
        }
        return nil
    }

    public func getSuggestions(urlc: URLComponents, levDistMax: Int = 4, numSuggestions: Int = 3) -> [IconMatch] {
        var suggestedIcons = [IconMatch]()
        let URI = URIParameters()

        if let label = URI.getLabel(from: urlc) {
            for icons in iconsBrand {
                let levDist = levDis(label.issuer.lowercased(), icons.name)
                if levDist < levDistMax {
                    let icon = IconMatch(name: icons.value, levDist: levDist)
                    suggestedIcons.append(icon)
                }
            }
        }

        suggestedIcons.sort { $0.levDist < $1.levDist }

        let remove = suggestedIcons.count - numSuggestions
        if remove > 0 {
            suggestedIcons.removeLast(remove)
        }

        return suggestedIcons
    }

    func levDis(_ w1: String, _ w2: String) -> Int {
        let empty = [Int](repeating:0, count: w2.count)
        var last = [Int](0...w2.count)

        for (i, char1) in w1.enumerated() {
            var cur = [i + 1] + empty
            for (j, char2) in w2.enumerated() {
                cur[j + 1] = char1 == char2 ? last[j] : Swift.min(last[j], last[j + 1], cur[j]) + 1
            }
            last = cur
        }
        return last.last!
    }

    let iconBGColorMapping = [
        "bitbucket": "#0747A6",
        "fa-bitbucket": "#0747A6",
        "facebook": "#3B5998",
        "fa-facebook": "#3B5998",
        "github": "#24292E",
        "fa-github": "#24292E",
        "gitlab": "#292961",
        "fa-gitlab": "#292961",
        "reddit": "#FF4500",
        "fa-reddit": "#FF4500",
        "redhat": "#FF0000",
        "fa-redhat": "#FF0000",
        "steam": "#242424",
        "fa-steam": "#242424",

    ]

    //  Android material design colors https:gist.github.com/daniellevass/b0b8cfa773488e138037
    let backgroundColors = [
        "#F44336",          // md_red_500
        "#673AB7",          // md_deep_purple_500
        "#03A9F4",          // md_light_blue_500
        "#4CAF50",          // md_green_500
        "#FF5722",          // md_deep_orange_500
        "#E91E63",          // md_pink_500
        "#3F51B5",          // md_indigo_500
        "#00BCD4",          // md_cyan_500
        "#8BC34A",          // md_light_green_500
        "#FFC107",          // md_amber_500
        "#795548",          // md_brown_500
        "#9C27B0",          // md_purple_500
        "#2196F3",          // md_blue_500
        "#009688",          // md_teal_500
        "#CDDC39",          // md_lime_500
        "#FF9800",          // md_orange_500
    ]

    let issuerBrandMapping = [
        "gitlab.com": "fa-gitlab"
    ]

    let faIconsSolid = [
        "ad": "fa-ad",
        "addressBook": "fa-address-book",
        "addressCard": "fa-address-card",
        "adjust": "fa-adjust",
        "airFreshener": "fa-air-freshener",
        "alignCenter": "fa-align-center",
        "alignJustify": "fa-align-justify",
        "alignLeft": "fa-align-left",
        "alignRight": "fa-align-right",
        "allergies": "fa-allergies",
        "ambulance": "fa-ambulance",
        "americanSignLanguageInterpreting": "fa-american-sign-language-interpreting",
        "anchor": "fa-anchor",
        "angleDoubleDown": "fa-angle-double-down",
        "angleDoubleLeft": "fa-angle-double-left",
        "angleDoubleRight": "fa-angle-double-right",
        "angleDoubleUp": "fa-angle-double-up",
        "angleDown": "fa-angle-down",
        "angleLeft": "fa-angle-left",
        "angleRight": "fa-angle-right",
        "angleUp": "fa-angle-up",
        "angry": "fa-angry",
        "ankh": "fa-ankh",
        "appleAlt": "fa-apple-alt",
        "archive": "fa-archive",
        "archway": "fa-archway",
        "arrowAltCircleDown": "fa-arrow-alt-circle-down",
        "arrowAltCircleLeft": "fa-arrow-alt-circle-left",
        "arrowAltCircleRight": "fa-arrow-alt-circle-right",
        "arrowAltCircleUp": "fa-arrow-alt-circle-up",
        "arrowCircleDown": "fa-arrow-circle-down",
        "arrowCircleLeft": "fa-arrow-circle-left",
        "arrowCircleRight": "fa-arrow-circle-right",
        "arrowCircleUp": "fa-arrow-circle-up",
        "arrowDown": "fa-arrow-down",
        "arrowLeft": "fa-arrow-left",
        "arrowRight": "fa-arrow-right",
        "arrowUp": "fa-arrow-up",
        "arrowsAlt": "fa-arrows-alt",
        "arrowsAltH": "fa-arrows-alt-h",
        "arrowsAltV": "fa-arrows-alt-v",
        "assistiveListeningSystems": "fa-assistive-listening-systems",
        "asterisk": "fa-asterisk",
        "at": "fa-at",
        "atlas": "fa-atlas",
        "atom": "fa-atom",
        "audioDescription": "fa-audio-description",
        "award": "fa-award",
        "baby": "fa-baby",
        "babyCarriage": "fa-baby-carriage",
        "backspace": "fa-backspace",
        "backward": "fa-backward",
        "bacon": "fa-bacon",
        "balanceScale": "fa-balance-scale",
        "balanceScaleLeft": "fa-balance-scale-left",
        "balanceScaleRight": "fa-balance-scale-right",
        "ban": "fa-ban",
        "bandAid": "fa-band-aid",
        "barcode": "fa-barcode",
        "bars": "fa-bars",
        "baseballBall": "fa-baseball-ball",
        "basketballBall": "fa-basketball-ball",
        "bath": "fa-bath",
        "batteryEmpty": "fa-battery-empty",
        "batteryFull": "fa-battery-full",
        "batteryHalf": "fa-battery-half",
        "batteryQuarter": "fa-battery-quarter",
        "batteryThreeQuarters": "fa-battery-three-quarters",
        "bed": "fa-bed",
        "beer": "fa-beer",
        "bell": "fa-bell",
        "bellSlash": "fa-bell-slash",
        "bezierCurve": "fa-bezier-curve",
        "bible": "fa-bible",
        "bicycle": "fa-bicycle",
        "biking": "fa-biking",
        "binoculars": "fa-binoculars",
        "biohazard": "fa-biohazard",
        "birthdayCake": "fa-birthday-cake",
        "blender": "fa-blender",
        "blenderPhone": "fa-blender-phone",
        "blind": "fa-blind",
        "blog": "fa-blog",
        "bold": "fa-bold",
        "bolt": "fa-bolt",
        "bomb": "fa-bomb",
        "bone": "fa-bone",
        "bong": "fa-bong",
        "book": "fa-book",
        "bookDead": "fa-book-dead",
        "bookMedical": "fa-book-medical",
        "bookOpen": "fa-book-open",
        "bookReader": "fa-book-reader",
        "bookmark": "fa-bookmark",
        "borderAll": "fa-border-all",
        "borderNone": "fa-border-none",
        "borderStyle": "fa-border-style",
        "bowlingBall": "fa-bowling-ball",
        "box": "fa-box",
        "boxOpen": "fa-box-open",
        "boxes": "fa-boxes",
        "braille": "fa-braille",
        "brain": "fa-brain",
        "breadSlice": "fa-bread-slice",
        "brief": "fa-briefcase",
        "briefcaseMedical": "fa-briefcase-medical",
        "broadcastTower": "fa-broadcast-tower",
        "broom": "fa-broom",
        "brush": "fa-brush",
        "bug": "fa-bug",
        "building": "fa-building",
        "bullhorn": "fa-bullhorn",
        "bullseye": "fa-bullseye",
        "burn": "fa-burn",
        "bus": "fa-bus",
        "busAlt": "fa-bus-alt",
        "businessTime": "fa-business-time",
        "calculator": "fa-calculator",
        "calendar": "fa-calendar",
        "calendarAlt": "fa-calendar-alt",
        "calendarCheck": "fa-calendar-check",
        "calendarDay": "fa-calendar-day",
        "calendarMinus": "fa-calendar-minus",
        "calendarPlus": "fa-calendar-plus",
        "calendarTimes": "fa-calendar-times",
        "calendarWeek": "fa-calendar-week",
        "camera": "fa-camera",
        "cameraRetro": "fa-camera-retro",
        "campground": "fa-campground",
        "candyCane": "fa-candy-cane",
        "cannabis": "fa-cannabis",
        "capsules": "fa-capsules",
        "car": "fa-car",
        "carAlt": "fa-car-alt",
        "carBattery": "fa-car-battery",
        "carCrash": "fa-car-crash",
        "carSide": "fa-car-side",
        "caretDown": "fa-caret-down",
        "caretLeft": "fa-caret-left",
        "caretRight": "fa-caret-right",
        "caretSquareDown": "fa-caret-square-down",
        "caretSquareLeft": "fa-caret-square-left",
        "caretSquareRight": "fa-caret-square-right",
        "caretSquareUp": "fa-caret-square-up",
        "caretUp": "fa-caret-up",
        "carrot": "fa-carrot",
        "cartArrowDown": "fa-cart-arrow-down",
        "cartPlus": "fa-cart-plus",
        "cashRegister": "fa-cash-register",
        "cat": "fa-cat",
        "certificate": "fa-certificate",
        "chair": "fa-chair",
        "chalkboard": "fa-chalkboard",
        "chalkboardTeacher": "fa-chalkboard-teacher",
        "chargingStation": "fa-charging-station",
        "chartArea": "fa-chart-area",
        "chartBar": "fa-chart-bar",
        "chartLine": "fa-chart-line",
        "chartPie": "fa-chart-pie",
        "check": "fa-check",
        "checkCircle": "fa-check-circle",
        "checkDouble": "fa-check-double",
        "checkSquare": "fa-check-square",
        "cheese": "fa-cheese",
        "chess": "fa-chess",
        "chessBishop": "fa-chess-bishop",
        "chessBoard": "fa-chess-board",
        "chessKing": "fa-chess-king",
        "chessKnight": "fa-chess-knight",
        "chessPawn": "fa-chess-pawn",
        "chessQueen": "fa-chess-queen",
        "chessRook": "fa-chess-rook",
        "chevronCircleDown": "fa-chevron-circle-down",
        "chevronCircleLeft": "fa-chevron-circle-left",
        "chevronCircleRight": "fa-chevron-circle-right",
        "chevronCircleUp": "fa-chevron-circle-up",
        "chevronDown": "fa-chevron-down",
        "chevronLeft": "fa-chevron-left",
        "chevronRight": "fa-chevron-right",
        "chevronUp": "fa-chevron-up",
        "child": "fa-child",
        "church": "fa-church",
        "circle": "fa-circle",
        "circleNotch": "fa-circle-notch",
        "city": "fa-city",
        "clinicMedical": "fa-clinic-medical",
        "clipboard": "fa-clipboard",
        "clipboardCheck": "fa-clipboard-check",
        "clipboardList": "fa-clipboard-list",
        "clock": "fa-clock",
        "clone": "fa-clone",
        "closedCaptioning": "fa-closed-captioning",
        "cloud": "fa-cloud",
        "cloudDownloadAlt": "fa-cloud-download-alt",
        "cloudMeatball": "fa-cloud-meatball",
        "cloudMoon": "fa-cloud-moon",
        "cloudMoonRain": "fa-cloud-moon-rain",
        "cloudRain": "fa-cloud-rain",
        "cloudShowersHeavy": "fa-cloud-showers-heavy",
        "cloudSun": "fa-cloud-sun",
        "cloudSunRain": "fa-cloud-sun-rain",
        "cloudUploadAlt": "fa-cloud-upload-alt",
        "cocktail": "fa-cocktail",
        "code": "fa-code",
        "codeBranch": "fa-code-branch",
        "coffee": "fa-coffee",
        "cog": "fa-cog",
        "cogs": "fa-cogs",
        "coins": "fa-coins",
        "columns": "fa-columns",
        "comment": "fa-comment",
        "commentAlt": "fa-comment-alt",
        "commentDollar": "fa-comment-dollar",
        "commentDots": "fa-comment-dots",
        "commentMedical": "fa-comment-medical",
        "commentSlash": "fa-comment-slash",
        "comments": "fa-comments",
        "commentsDollar": "fa-comments-dollar",
        "compactDisc": "fa-compact-disc",
        "compass": "fa-compass",
        "compress": "fa-compress",
        "compressArrowsAlt": "fa-compress-arrows-alt",
        "conciergeBell": "fa-concierge-bell",
        "cookie": "fa-cookie",
        "cookieBite": "fa-cookie-bite",
        "copy": "fa-copy",
        "copyright": "fa-copyright",
        "couch": "fa-couch",
        "creditCard": "fa-credit-card",
        "crop": "fa-crop",
        "cropAlt": "fa-crop-alt",
        "cross": "fa-cross",
        "crosshairs": "fa-crosshairs",
        "crow": "fa-crow",
        "crown": "fa-crown",
        "crutch": "fa-crutch",
        "cube": "fa-cube",
        "cubes": "fa-cubes",
        "cut": "fa-cut",
        "database": "fa-database",
        "deaf": "fa-deaf",
        "democrat": "fa-democrat",
        "desktop": "fa-desktop",
        "dharmachakra": "fa-dharmachakra",
        "diagnoses": "fa-diagnoses",
        "dice": "fa-dice",
        "diceD20": "fa-dice-d20",
        "diceD6": "fa-dice-d6",
        "diceFive": "fa-dice-five",
        "diceFour": "fa-dice-four",
        "diceOne": "fa-dice-one",
        "diceSix": "fa-dice-six",
        "diceThree": "fa-dice-three",
        "diceTwo": "fa-dice-two",
        "digitalTachograph": "fa-digital-tachograph",
        "directions": "fa-directions",
        "divide": "fa-divide",
        "dizzy": "fa-dizzy",
        "dna": "fa-dna",
        "dog": "fa-dog",
        "dollarSign": "fa-dollar-sign",
        "dolly": "fa-dolly",
        "dollyFlatbed": "fa-dolly-flatbed",
        "donate": "fa-donate",
        "doorClosed": "fa-door-closed",
        "doorOpen": "fa-door-open",
        "dotCircle": "fa-dot-circle",
        "dove": "fa-dove",
        "download": "fa-download",
        "draftingCompass": "fa-drafting-compass",
        "dragon": "fa-dragon",
        "drawPolygon": "fa-draw-polygon",
        "drum": "fa-drum",
        "drumSteelpan": "fa-drum-steelpan",
        "drumstickBite": "fa-drumstick-bite",
        "dumbbell": "fa-dumbbell",
        "dumpster": "fa-dumpster",
        "dumpsterFire": "fa-dumpster-fire",
        "dungeon": "fa-dungeon",
        "edit": "fa-edit",
        "egg": "fa-egg",
        "eject": "fa-eject",
        "ellipsisH": "fa-ellipsis-h",
        "ellipsisV": "fa-ellipsis-v",
        "envelope": "fa-envelope",
        "envelopeOpen": "fa-envelope-open",
        "envelopeOpenText": "fa-envelope-open-text",
        "envelopeSquare": "fa-envelope-square",
        "equals": "fa-equals",
        "eraser": "fa-eraser",
        "ethernet": "fa-ethernet",
        "euroSign": "fa-euro-sign",
        "exchangeAlt": "fa-exchange-alt",
        "exclamation": "fa-exclamation",
        "exclamationCircle": "fa-exclamation-circle",
        "exclamationTriangle": "fa-exclamation-triangle",
        "expand": "fa-expand",
        "expandArrowsAlt": "fa-expand-arrows-alt",
        "externalLinkAlt": "fa-external-link-alt",
        "externalLinkSquareAlt": "fa-external-link-square-alt",
        "eye": "fa-eye",
        "eyeDropper": "fa-eye-dropper",
        "eyeSlash": "fa-eye-slash",
        "fan": "fa-fan",
        "fastBackward": "fa-fast-backward",
        "fastForward": "fa-fast-forward",
        "fax": "fa-fax",
        "feather": "fa-feather",
        "featherAlt": "fa-feather-alt",
        "female": "fa-female",
        "fighterJet": "fa-fighter-jet",
        "file": "fa-file",
        "fileAlt": "fa-file-alt",
        "fileArchive": "fa-file-archive",
        "fileAudio": "fa-file-audio",
        "fileCode": "fa-file-code",
        "fileContract": "fa-file-contract",
        "fileCsv": "fa-file-csv",
        "fileDownload": "fa-file-download",
        "fileExcel": "fa-file-excel",
        "fileExport": "fa-file-export",
        "fileImage": "fa-file-image",
        "fileImport": "fa-file-import",
        "fileInvoice": "fa-file-invoice",
        "fileInvoiceDollar": "fa-file-invoice-dollar",
        "fileMedical": "fa-file-medical",
        "fileMedicalAlt": "fa-file-medical-alt",
        "filePdf": "fa-file-pdf",
        "filePowerpoint": "fa-file-powerpoint",
        "filePrescription": "fa-file-prescription",
        "fileSignature": "fa-file-signature",
        "fileUpload": "fa-file-upload",
        "fileVideo": "fa-file-video",
        "fileWord": "fa-file-word",
        "fill": "fa-fill",
        "fillDrip": "fa-fill-drip",
        "film": "fa-film",
        "filter": "fa-filter",
        "fingerprint": "fa-fingerprint",
        "fire": "fa-fire",
        "fireAlt": "fa-fire-alt",
        "fireExtinguisher": "fa-fire-extinguisher",
        "firstAid": "fa-first-aid",
        "fish": "fa-fish",
        "fistRaised": "fa-fist-raised",
        "flag": "fa-flag",
        "flagCheckered": "fa-flag-checkered",
        "flagUsa": "fa-flag-usa",
        "flask": "fa-flask",
        "flushed": "fa-flushed",
        "folder": "fa-folder",
        "folderMinus": "fa-folder-minus",
        "folderOpen": "fa-folder-open",
        "folderPlus": "fa-folder-plus",
        "font": "fa-font",
        "footballBall": "fa-football-ball",
        "forward": "fa-forward",
        "frog": "fa-frog",
        "frown": "fa-frown",
        "frownOpen": "fa-frown-open",
        "funnelDollar": "fa-funnel-dollar",
        "futbol": "fa-futbol",
        "gamepad": "fa-gamepad",
        "gasPump": "fa-gas-pump",
        "gavel": "fa-gavel",
        "gem": "fa-gem",
        "genderless": "fa-genderless",
        "ghost": "fa-ghost",
        "gift": "fa-gift",
        "gifts": "fa-gifts",
        "glassCheers": "fa-glass-cheers",
        "glassMartini": "fa-glass-martini",
        "glassMartiniAlt": "fa-glass-martini-alt",
        "glassWhiskey": "fa-glass-whiskey",
        "glasses": "fa-glasses",
        "globe": "fa-globe",
        "globeAfrica": "fa-globe-africa",
        "globeAmericas": "fa-globe-americas",
        "globeAsia": "fa-globe-asia",
        "globeEurope": "fa-globe-europe",
        "golfBall": "fa-golf-ball",
        "gopuram": "fa-gopuram",
        "graduationCap": "fa-graduation-cap",
        "greaterThan": "fa-greater-than",
        "greaterThanEqual": "fa-greater-than-equal",
        "grimace": "fa-grimace",
        "grin": "fa-grin",
        "grinAlt": "fa-grin-alt",
        "grinBeam": "fa-grin-beam",
        "grinBeamSweat": "fa-grin-beam-sweat",
        "grinHearts": "fa-grin-hearts",
        "grinSquint": "fa-grin-squint",
        "grinSquintTears": "fa-grin-squint-tears",
        "grinStars": "fa-grin-stars",
        "grinTears": "fa-grin-tears",
        "grinTongue": "fa-grin-tongue",
        "grinTongueSquint": "fa-grin-tongue-squint",
        "grinTongueWink": "fa-grin-tongue-wink",
        "grinWink": "fa-grin-wink",
        "gripHorizontal": "fa-grip-horizontal",
        "gripLines": "fa-grip-lines",
        "gripLinesVertical": "fa-grip-lines-vertical",
        "gripVertical": "fa-grip-vertical",
        "guitar": "fa-guitar",
        "hSquare": "fa-h-square",
        "hamburger": "fa-hamburger",
        "hammer": "fa-hammer",
        "hamsa": "fa-hamsa",
        "handHolding": "fa-hand-holding",
        "handHoldingHeart": "fa-hand-holding-heart",
        "handHoldingUsd": "fa-hand-holding-usd",
        "handLizard": "fa-hand-lizard",
        "handMiddleFinger": "fa-hand-middle-finger",
        "handPaper": "fa-hand-paper",
        "handPeace": "fa-hand-peace",
        "handPointDown": "fa-hand-point-down",
        "handPointLeft": "fa-hand-point-left",
        "handPointRight": "fa-hand-point-right",
        "handPointUp": "fa-hand-point-up",
        "handPointer": "fa-hand-pointer",
        "handRock": "fa-hand-rock",
        "handScissors": "fa-hand-scissors",
        "handSpock": "fa-hand-spock",
        "hands": "fa-hands",
        "handsHelping": "fa-hands-helping",
        "handshake": "fa-handshake",
        "hanukiah": "fa-hanukiah",
        "hardHat": "fa-hard-hat",
        "hashtag": "fa-hashtag",
        "hatWizard": "fa-hat-wizard",
        "haykal": "fa-haykal",
        "hdd": "fa-hdd",
        "heading": "fa-heading",
        "headphones": "fa-headphones",
        "headphonesAlt": "fa-headphones-alt",
        "headset": "fa-headset",
        "heart": "fa-heart",
        "heartBroken": "fa-heart-broken",
        "heartbeat": "fa-heartbeat",
        "helicopter": "fa-helicopter",
        "highlighter": "fa-highlighter",
        "hiking": "fa-hiking",
        "hippo": "fa-hippo",
        "history": "fa-history",
        "hockeyPuck": "fa-hockey-puck",
        "hollyBerry": "fa-holly-berry",
        "home": "fa-home",
        "horse": "fa-horse",
        "horseHead": "fa-horse-head",
        "hospital": "fa-hospital",
        "hospitalAlt": "fa-hospital-alt",
        "hospitalSymbol": "fa-hospital-symbol",
        "hotTub": "fa-hot-tub",
        "hotdog": "fa-hotdog",
        "hotel": "fa-hotel",
        "hourglass": "fa-hourglass",
        "hourglassEnd": "fa-hourglass-end",
        "hourglassHalf": "fa-hourglass-half",
        "hourglassStart": "fa-hourglass-start",
        "houseDamage": "fa-house-damage",
        "hryvnia": "fa-hryvnia",
        "iCursor": "fa-i-cursor",
        "iceCream": "fa-ice-cream",
        "icicles": "fa-icicles",
        "icons": "fa-icons",
        "idBadge": "fa-id-badge",
        "idCard": "fa-id-card",
        "idCardAlt": "fa-id-card-alt",
        "igloo": "fa-igloo",
        "image": "fa-image",
        "images": "fa-images",
        "inbox": "fa-inbox",
        "indent": "fa-indent",
        "industry": "fa-industry",
        "infinity": "fa-infinity",
        "info": "fa-info",
        "infoCircle": "fa-info-circle",
        "italic": "fa-italic",
        "jedi": "fa-jedi",
        "jenkins": "fa-jenkins",
        "joint": "fa-joint",
        "journalWhills": "fa-journal-whills",
        "kaaba": "fa-kaaba",
        "key": "fa-key",
        "keyboard": "fa-keyboard",
        "khanda": "fa-khanda",
        "kiss": "fa-kiss",
        "kissBeam": "fa-kiss-beam",
        "kissWinkHeart": "fa-kiss-wink-heart",
        "kiwiBird": "fa-kiwi-bird",
        "landmark": "fa-landmark",
        "language": "fa-language",
        "laptop": "fa-laptop",
        "laptopCode": "fa-laptop-code",
        "laptopMedical": "fa-laptop-medical",
        "laugh": "fa-laugh",
        "laughBeam": "fa-laugh-beam",
        "laughSquint": "fa-laugh-squint",
        "laughWink": "fa-laugh-wink",
        "layerGroup": "fa-layer-group",
        "leaf": "fa-leaf",
        "lemon": "fa-lemon",
        "lessThan": "fa-less-than",
        "lessThanEqual": "fa-less-than-equal",
        "levelDownAlt": "fa-level-down-alt",
        "levelUpAlt": "fa-level-up-alt",
        "lifeRing": "fa-life-ring",
        "lightbulb": "fa-lightbulb",
        "link": "fa-link",
        "liraSign": "fa-lira-sign",
        "list": "fa-list",
        "listAlt": "fa-list-alt",
        "listOl": "fa-list-ol",
        "listUl": "fa-list-ul",
        "locationArrow": "fa-location-arrow",
        "lock": "fa-lock",
        "lockOpen": "fa-lock-open",
        "longArrowAltDown": "fa-long-arrow-alt-down",
        "longArrowAltLeft": "fa-long-arrow-alt-left",
        "longArrowAltRight": "fa-long-arrow-alt-right",
        "longArrowAltUp": "fa-long-arrow-alt-up",
        "lowVision": "fa-low-vision",
        "luggageCart": "fa-luggage-cart",
        "magic": "fa-magic",
        "magnet": "fa-magnet",
        "mailBulk": "fa-mail-bulk",
        "male": "fa-male",
        "map": "fa-map",
        "mapMarked": "fa-map-marked",
        "mapMarkedAlt": "fa-map-marked-alt",
        "mapMarker": "fa-map-marker",
        "mapMarkerAlt": "fa-map-marker-alt",
        "mapPin": "fa-map-pin",
        "mapSigns": "fa-map-signs",
        "marker": "fa-marker",
        "mars": "fa-mars",
        "marsDouble": "fa-mars-double",
        "marsStroke": "fa-mars-stroke",
        "marsStrokeH": "fa-mars-stroke-h",
        "marsStrokeV": "fa-mars-stroke-v",
        "mask": "fa-mask",
        "medal": "fa-medal",
        "medkit": "fa-medkit",
        "meh": "fa-meh",
        "mehBlank": "fa-meh-blank",
        "mehRollingEyes": "fa-meh-rolling-eyes",
        "memory": "fa-memory",
        "menorah": "fa-menorah",
        "mercury": "fa-mercury",
        "meteor": "fa-meteor",
        "microchip": "fa-microchip",
        "microphone": "fa-microphone",
        "microphoneAlt": "fa-microphone-alt",
        "microphoneAltSlash": "fa-microphone-alt-slash",
        "microphoneSlash": "fa-microphone-slash",
        "microscope": "fa-microscope",
        "minus": "fa-minus",
        "minusCircle": "fa-minus-circle",
        "minusSquare": "fa-minus-square",
        "mitten": "fa-mitten",
        "mobile": "fa-mobile",
        "mobileAlt": "fa-mobile-alt",
        "moneyBill": "fa-money-bill",
        "moneyBillAlt": "fa-money-bill-alt",
        "moneyBillWave": "fa-money-bill-wave",
        "moneyBillWaveAlt": "fa-money-bill-wave-alt",
        "moneyCheck": "fa-money-check",
        "moneyCheckAlt": "fa-money-check-alt",
        "monument": "fa-monument",
        "moon": "fa-moon",
        "mortarPestle": "fa-mortar-pestle",
        "mosque": "fa-mosque",
        "motorcycle": "fa-motorcycle",
        "mountain": "fa-mountain",
        "mousePointer": "fa-mouse-pointer",
        "mugHot": "fa-mug-hot",
        "music": "fa-music",
        "networkWired": "fa-network-wired",
        "neuter": "fa-neuter",
        "newspaper": "fa-newspaper",
        "notEqual": "fa-not-equal",
        "notesMedical": "fa-notes-medical",
        "objectGroup": "fa-object-group",
        "objectUngroup": "fa-object-ungroup",
        "oilCan": "fa-oil-can",
        "om": "fa-om",
        "otter": "fa-otter",
        "outdent": "fa-outdent",
        "pager": "fa-pager",
        "paintBrush": "fa-paint-brush",
        "paintRoller": "fa-paint-roller",
        "palette": "fa-palette",
        "pallet": "fa-pallet",
        "paperPlane": "fa-paper-plane",
        "paperclip": "fa-paperclip",
        "parachuteBox": "fa-parachute-box",
        "paragraph": "fa-paragraph",
        "parking": "fa-parking",
        "passport": "fa-passport",
        "pastafarianism": "fa-pastafarianism",
        "paste": "fa-paste",
        "pause": "fa-pause",
        "pauseCircle": "fa-pause-circle",
        "paw": "fa-paw",
        "peace": "fa-peace",
        "pen": "fa-pen",
        "penAlt": "fa-pen-alt",
        "penFancy": "fa-pen-fancy",
        "penNib": "fa-pen-nib",
        "penSquare": "fa-pen-square",
        "pencilAlt": "fa-pencil-alt",
        "pencilRuler": "fa-pencil-ruler",
        "peopleCarry": "fa-people-carry",
        "pepperHot": "fa-pepper-hot",
        "percent": "fa-percent",
        "percentage": "fa-percentage",
        "personBooth": "fa-person-booth",
        "phone": "fa-phone",
        "phoneAlt": "fa-phone-alt",
        "phoneSlash": "fa-phone-slash",
        "phoneSquare": "fa-phone-square",
        "phoneSquareAlt": "fa-phone-square-alt",
        "phoneVolume": "fa-phone-volume",
        "photoVideo": "fa-photo-video",
        "piggyBank": "fa-piggy-bank",
        "pills": "fa-pills",
        "pizzaSlice": "fa-pizza-slice",
        "placeOfWorship": "fa-place-of-worship",
        "plane": "fa-plane",
        "planeArrival": "fa-plane-arrival",
        "planeDeparture": "fa-plane-departure",
        "play": "fa-play",
        "playCircle": "fa-play-circle",
        "plug": "fa-plug",
        "plus": "fa-plus",
        "plusCircle": "fa-plus-circle",
        "plusSquare": "fa-plus-square",
        "podcast": "fa-podcast",
        "poll": "fa-poll",
        "pollH": "fa-poll-h",
        "poo": "fa-poo",
        "pooStorm": "fa-poo-storm",
        "poop": "fa-poop",
        "portrait": "fa-portrait",
        "poundSign": "fa-pound-sign",
        "powerOff": "fa-power-off",
        "pray": "fa-pray",
        "prayingHands": "fa-praying-hands",
        "prescription": "fa-prescription",
        "prescriptionBottle": "fa-prescription-bottle",
        "prescriptionBottleAlt": "fa-prescription-bottle-alt",
        "print": "fa-print",
        "procedures": "fa-procedures",
        "projectDiagram": "fa-project-diagram",
        "puzzlePiece": "fa-puzzle-piece",
        "qrcode": "fa-qrcode",
        "question": "fa-question",
        "questionCircle": "fa-question-circle",
        "quidditch": "fa-quidditch",
        "quoteLeft": "fa-quote-left",
        "quoteRight": "fa-quote-right",
        "quran": "fa-quran",
        "radiation": "fa-radiation",
        "radiationAlt": "fa-radiation-alt",
        "rainbow": "fa-rainbow",
        "random": "fa-random",
        "receipt": "fa-receipt",
        "recycle": "fa-recycle",
        "redo": "fa-redo",
        "redoAlt": "fa-redo-alt",
        "registered": "fa-registered",
        "removeFormat": "fa-remove-format",
        "reply": "fa-reply",
        "replyAll": "fa-reply-all",
        "republican": "fa-republican",
        "restroom": "fa-restroom",
        "retweet": "fa-retweet",
        "ribbon": "fa-ribbon",
        "ring": "fa-ring",
        "road": "fa-road",
        "robot": "fa-robot",
        "rocket": "fa-rocket",
        "route": "fa-route",
        "rss": "fa-rss",
        "rssSquare": "fa-rss-square",
        "rubleSign": "fa-ruble-sign",
        "ruler": "fa-ruler",
        "rulerCombined": "fa-ruler-combined",
        "rulerHorizontal": "fa-ruler-horizontal",
        "rulerVertical": "fa-ruler-vertical",
        "running": "fa-running",
        "rupeeSign": "fa-rupee-sign",
        "sadCry": "fa-sad-cry",
        "sadTear": "fa-sad-tear",
        "satellite": "fa-satellite",
        "satelliteDish": "fa-satellite-dish",
        "save": "fa-save",
        "school": "fa-school",
        "screwdriver": "fa-screwdriver",
        "scroll": "fa-scroll",
        "sdCard": "fa-sd-card",
        "search": "fa-search",
        "searchDollar": "fa-search-dollar",
        "searchLocation": "fa-search-location",
        "searchMinus": "fa-search-minus",
        "searchPlus": "fa-search-plus",
        "seedling": "fa-seedling",
        "server": "fa-server",
        "shapes": "fa-shapes",
        "share": "fa-share",
        "shareAlt": "fa-share-alt",
        "shareAltSquare": "fa-share-alt-square",
        "shareSquare": "fa-share-square",
        "shekelSign": "fa-shekel-sign",
        "shieldAlt": "fa-shield-alt",
        "ship": "fa-ship",
        "shippingFast": "fa-shipping-fast",
        "shoePrints": "fa-shoe-prints",
        "shoppingBag": "fa-shopping-bag",
        "shoppingBasket": "fa-shopping-basket",
        "shoppingCart": "fa-shopping-cart",
        "shower": "fa-shower",
        "shuttleVan": "fa-shuttle-van",
        "sign": "fa-sign",
        "signInAlt": "fa-sign-in-alt",
        "signLanguage": "fa-sign-language",
        "signOutAlt": "fa-sign-out-alt",
        "signal": "fa-signal",
        "signature": "fa-signature",
        "simCard": "fa-sim-card",
        "sitemap": "fa-sitemap",
        "skating": "fa-skating",
        "skiing": "fa-skiing",
        "skiingNordic": "fa-skiing-nordic",
        "skull": "fa-skull",
        "skullCrossbones": "fa-skull-crossbones",
        "slash": "fa-slash",
        "sleigh": "fa-sleigh",
        "slidersH": "fa-sliders-h",
        "smile": "fa-smile",
        "smileBeam": "fa-smile-beam",
        "smileWink": "fa-smile-wink",
        "smog": "fa-smog",
        "smoking": "fa-smoking",
        "smokingBan": "fa-smoking-ban",
        "sms": "fa-sms",
        "snowboarding": "fa-snowboarding",
        "snowflake": "fa-snowflake",
        "snowman": "fa-snowman",
        "snowplow": "fa-snowplow",
        "socks": "fa-socks",
        "solarPanel": "fa-solar-panel",
        "sort": "fa-sort",
        "sortAlphaDown": "fa-sort-alpha-down",
        "sortAlphaDownAlt": "fa-sort-alpha-down-alt",
        "sortAlphaUp": "fa-sort-alpha-up",
        "sortAlphaUpAlt": "fa-sort-alpha-up-alt",
        "sortAmountDown": "fa-sort-amount-down",
        "sortAmountDownAlt": "fa-sort-amount-down-alt",
        "sortAmountUp": "fa-sort-amount-up",
        "sortAmountUpAlt": "fa-sort-amount-up-alt",
        "sortDown": "fa-sort-down",
        "sortNumericDown": "fa-sort-numeric-down",
        "sortNumericDownAlt": "fa-sort-numeric-down-alt",
        "sortNumericUp": "fa-sort-numeric-up",
        "sortNumericUpAlt": "fa-sort-numeric-up-alt",
        "sortUp": "fa-sort-up",
        "spa": "fa-spa",
        "spaceShuttle": "fa-space-shuttle",
        "spellCheck": "fa-spell-check",
        "spider": "fa-spider",
        "spinner": "fa-spinner",
        "splotch": "fa-splotch",
        "sprayCan": "fa-spray-can",
        "square": "fa-square",
        "squareFull": "fa-square-full",
        "squareRootAlt": "fa-square-root-alt",
        "stamp": "fa-stamp",
        "star": "fa-star",
        "starAndCrescent": "fa-star-and-crescent",
        "starHalf": "fa-star-half",
        "starHalfAlt": "fa-star-half-alt",
        "starOfDavid": "fa-star-of-david",
        "starOfLife": "fa-star-of-life",
        "stepBackward": "fa-step-backward",
        "stepForward": "fa-step-forward",
        "stethoscope": "fa-stethoscope",
        "stickyNote": "fa-sticky-note",
        "stop": "fa-stop",
        "stopCircle": "fa-stop-circle",
        "stopwatch": "fa-stopwatch",
        "store": "fa-store",
        "storeAlt": "fa-store-alt",
        "stream": "fa-stream",
        "streetView": "fa-street-view",
        "strikethrough": "fa-strikethrough",
        "stroopwafel": "fa-stroopwafel",
        "`subscript`": "fa-subscript",
        "subway": "fa-subway",
        "suit": "fa-suitcase",
        "suitcaseRolling": "fa-suitcase-rolling",
        "sun": "fa-sun",
        "superscript": "fa-superscript",
        "supple": "fa-supple",
        "surprise": "fa-surprise",
        "swatchbook": "fa-swatchbook",
        "swimmer": "fa-swimmer",
        "swimmingPool": "fa-swimming-pool",
        "synagogue": "fa-synagogue",
        "sync": "fa-sync",
        "syncAlt": "fa-sync-alt",
        "syringe": "fa-syringe",
        "table": "fa-table",
        "tableTennis": "fa-table-tennis",
        "tablet": "fa-tablet",
        "tabletAlt": "fa-tablet-alt",
        "tablets": "fa-tablets",
        "tachometerAlt": "fa-tachometer-alt",
        "tag": "fa-tag",
        "tags": "fa-tags",
        "tape": "fa-tape",
        "tasks": "fa-tasks",
        "taxi": "fa-taxi",
        "teeth": "fa-teeth",
        "teethOpen": "fa-teeth-open",
        "temperatureHigh": "fa-temperature-high",
        "temperatureLow": "fa-temperature-low",
        "tenge": "fa-tenge",
        "terminal": "fa-terminal",
        "textHeight": "fa-text-height",
        "textWidth": "fa-text-width",
        "th": "fa-th",
        "thLarge": "fa-th-large",
        "thList": "fa-th-list",
        "theaterMasks": "fa-theater-masks",
        "thermometer": "fa-thermometer",
        "thermometerEmpty": "fa-thermometer-empty",
        "thermometerFull": "fa-thermometer-full",
        "thermometerHalf": "fa-thermometer-half",
        "thermometerQuarter": "fa-thermometer-quarter",
        "thermometerThreeQuarters": "fa-thermometer-three-quarters",
        "thumbsDown": "fa-thumbs-down",
        "thumbsUp": "fa-thumbs-up",
        "thumbtack": "fa-thumbtack",
        "ticketAlt": "fa-ticket-alt",
        "times": "fa-times",
        "timesCircle": "fa-times-circle",
        "tint": "fa-tint",
        "tintSlash": "fa-tint-slash",
        "tired": "fa-tired",
        "toggleOff": "fa-toggle-off",
        "toggleOn": "fa-toggle-on",
        "toilet": "fa-toilet",
        "toiletPaper": "fa-toilet-paper",
        "toolbox": "fa-toolbox",
        "tools": "fa-tools",
        "tooth": "fa-tooth",
        "torah": "fa-torah",
        "toriiGate": "fa-torii-gate",
        "tractor": "fa-tractor",
        "trademark": "fa-trademark",
        "trafficLight": "fa-traffic-light",
        "train": "fa-train",
        "tram": "fa-tram",
        "transgender": "fa-transgender",
        "transgenderAlt": "fa-transgender-alt",
        "trash": "fa-trash",
        "trashAlt": "fa-trash-alt",
        "trashRestore": "fa-trash-restore",
        "trashRestoreAlt": "fa-trash-restore-alt",
        "tree": "fa-tree",
        "trophy": "fa-trophy",
        "truck": "fa-truck",
        "truckLoading": "fa-truck-loading",
        "truckMonster": "fa-truck-monster",
        "truckMoving": "fa-truck-moving",
        "truckPickup": "fa-truck-pickup",
        "tshirt": "fa-tshirt",
        "tty": "fa-tty",
        "tv": "fa-tv",
        "umbrella": "fa-umbrella",
        "umbrellaBeach": "fa-umbrella-beach",
        "underline": "fa-underline",
        "undo": "fa-undo",
        "undoAlt": "fa-undo-alt",
        "universalAccess": "fa-universal-access",
        "university": "fa-university",
        "unlink": "fa-unlink",
        "unlock": "fa-unlock",
        "unlockAlt": "fa-unlock-alt",
        "upload": "fa-upload",
        "user": "fa-user",
        "userAlt": "fa-user-alt",
        "userAltSlash": "fa-user-alt-slash",
        "userAstronaut": "fa-user-astronaut",
        "userCheck": "fa-user-check",
        "userCircle": "fa-user-circle",
        "userClock": "fa-user-clock",
        "userCog": "fa-user-cog",
        "userEdit": "fa-user-edit",
        "userFriends": "fa-user-friends",
        "userGraduate": "fa-user-graduate",
        "userInjured": "fa-user-injured",
        "userLock": "fa-user-lock",
        "userMd": "fa-user-md",
        "userMinus": "fa-user-minus",
        "userNinja": "fa-user-ninja",
        "userNurse": "fa-user-nurse",
        "userPlus": "fa-user-plus",
        "userSecret": "fa-user-secret",
        "userShield": "fa-user-shield",
        "userSlash": "fa-user-slash",
        "userTag": "fa-user-tag",
        "userTie": "fa-user-tie",
        "userTimes": "fa-user-times",
        "users": "fa-users",
        "usersCog": "fa-users-cog",
        "utensilSpoon": "fa-utensil-spoon",
        "utensils": "fa-utensils",
        "vectorSquare": "fa-vector-square",
        "venus": "fa-venus",
        "venusDouble": "fa-venus-double",
        "venusMars": "fa-venus-mars",
        "vial": "fa-vial",
        "vials": "fa-vials",
        "video": "fa-video",
        "videoSlash": "fa-video-slash",
        "vihara": "fa-vihara",
        "voicemail": "fa-voicemail",
        "volleyballBall": "fa-volleyball-ball",
        "volumeDown": "fa-volume-down",
        "volumeMute": "fa-volume-mute",
        "volumeOff": "fa-volume-off",
        "volumeUp": "fa-volume-up",
        "voteYea": "fa-vote-yea",
        "vrCardboard": "fa-vr-cardboard",
        "walking": "fa-walking",
        "wallet": "fa-wallet",
        "warehouse": "fa-warehouse",
        "water": "fa-water",
        "waveSquare": "fa-wave-square",
        "weight": "fa-weight",
        "weightHanging": "fa-weight-hanging",
        "wheelchair": "fa-wheelchair",
        "wifi": "fa-wifi",
        "wind": "fa-wind",
        "windowClose": "fa-window-close",
        "windowMaximize": "fa-window-maximize",
        "windowMinimize": "fa-window-minimize",
        "windowRestore": "fa-window-restore",
        "wineBottle": "fa-wine-bottle",
        "wineGlass": "fa-wine-glass",
        "wineGlassAlt": "fa-wine-glass-alt",
        "wonSign": "fa-won-sign",
        "wrench": "fa-wrench",
        "xRay": "fa-x-ray",
        "yenSign": "fa-yen-sign",
        "yinYang": "fa-yin-yang",
    ]

    let faIconsBrands = [
        "fiveHundredPixels": "fa-500px",
        "accessibleIcon": "fa-accessible-icon",
        "accusoft": "fa-accusoft",
        "acquisitionsIncorporated": "fa-acquisitions-incorporated",
        "adn": "fa-adn",
        "adobe": "fa-adobe",
        "adversal": "fa-adversal",
        "affiliatetheme": "fa-affiliatetheme",
        "airbnb": "fa-airbnb",
        "algolia": "fa-algolia",
        "alipay": "fa-alipay",
        "amazon": "fa-amazon",
        "amazonPay": "fa-amazon-pay",
        "amilia": "fa-amilia",
        "android": "fa-android",
        "angellist": "fa-angellist",
        "angrycreative": "fa-angrycreative",
        "angular": "fa-angular",
        "appStore": "fa-app-store",
        "appStoreIos": "fa-app-store-ios",
        "apper": "fa-apper",
        "apple": "fa-apple",
        "applePay": "fa-apple-pay",
        "artstation": "fa-artstation",
        "asymmetrik": "fa-asymmetrik",
        "atlassian": "fa-atlassian",
        "audible": "fa-audible",
        "autoprefixer": "fa-autoprefixer",
        "avianex": "fa-avianex",
        "aviato": "fa-aviato",
        "aws": "fa-aws",
        "bandcamp": "fa-bandcamp",
        "battleNet": "fa-battle-net",
        "behance": "fa-behance",
        "behanceSquare": "fa-behance-square",
        "bimobject": "fa-bimobject",
        "bitbucket": "fa-bitbucket",
        "bitcoin": "fa-bitcoin",
        "bity": "fa-bity",
        "blackTie": "fa-black-tie",
        "blackberry": "fa-blackberry",
        "blogger": "fa-blogger",
        "bloggerB": "fa-blogger-b",
        "bluetooth": "fa-bluetooth",
        "bluetoothB": "fa-bluetooth-b",
        "bootstrap": "fa-bootstrap",
        "btc": "fa-btc",
        "buffer": "fa-buffer",
        "buromobelexperte": "fa-buromobelexperte",
        "buysellads": "fa-buysellads",
        "canadianMapleLeaf": "fa-canadian-maple-leaf",
        "ccAmazonPay": "fa-cc-amazon-pay",
        "ccAmex": "fa-cc-amex",
        "ccApplePay": "fa-cc-apple-pay",
        "ccDinersClub": "fa-cc-diners-club",
        "ccDiscover": "fa-cc-discover",
        "ccJcb": "fa-cc-jcb",
        "ccMastercard": "fa-cc-mastercard",
        "ccPaypal": "fa-cc-paypal",
        "ccStripe": "fa-cc-stripe",
        "ccVisa": "fa-cc-visa",
        "centercode": "fa-centercode",
        "centos": "fa-centos",
        "chrome": "fa-chrome",
        "chromecast": "fa-chromecast",
        "cloudscale": "fa-cloudscale",
        "cloudsmith": "fa-cloudsmith",
        "cloudversify": "fa-cloudversify",
        "codepen": "fa-codepen",
        "codiepie": "fa-codiepie",
        "confluence": "fa-confluence",
        "connectdevelop": "fa-connectdevelop",
        "contao": "fa-contao",
        "cottonBureau": "fa-cotton-bureau",
        "cpanel": "fa-cpanel",
        "creativeCommons": "fa-creative-commons",
        "creativeCommonsBy": "fa-creative-commons-by",
        "creativeCommonsNc": "fa-creative-commons-nc",
        "creativeCommonsNcEu": "fa-creative-commons-nc-eu",
        "creativeCommonsNcJp": "fa-creative-commons-nc-jp",
        "creativeCommonsNd": "fa-creative-commons-nd",
        "creativeCommonsPd": "fa-creative-commons-pd",
        "creativeCommonsPdAlt": "fa-creative-commons-pd-alt",
        "creativeCommonsRemix": "fa-creative-commons-remix",
        "creativeCommonsSa": "fa-creative-commons-sa",
        "creativeCommonsSampling": "fa-creative-commons-sampling",
        "creativeCommonsSamplingPlus": "fa-creative-commons-sampling-plus",
        "creativeCommonsShare": "fa-creative-commons-share",
        "creativeCommonsZero": "fa-creative-commons-zero",
        "criticalRole": "fa-critical-role",
        "css3": "fa-css3",
        "css3Alt": "fa-css3-alt",
        "cuttlefish": "fa-cuttlefish",
        "dAndD": "fa-d-and-d",
        "dAndDBeyond": "fa-d-and-d-beyond",
        "dashcube": "fa-dashcube",
        "delicious": "fa-delicious",
        "deploydog": "fa-deploydog",
        "deskpro": "fa-deskpro",
        "dev": "fa-dev",
        "deviantart": "fa-deviantart",
        "dhl": "fa-dhl",
        "diaspora": "fa-diaspora",
        "digg": "fa-digg",
        "digitalOcean": "fa-digital-ocean",
        "discord": "fa-discord",
        "discourse": "fa-discourse",
        "dochub": "fa-dochub",
        "docker": "fa-docker",
        "draft2digital": "fa-draft2digital",
        "dribbble": "fa-dribbble",
        "dribbbleSquare": "fa-dribbble-square",
        "dropbox": "fa-dropbox",
        "drupal": "fa-drupal",
        "dyalog": "fa-dyalog",
        "earlybirds": "fa-earlybirds",
        "ebay": "fa-ebay",
        "edge": "fa-edge",
        "elementor": "fa-elementor",
        "ello": "fa-ello",
        "ember": "fa-ember",
        "empire": "fa-empire",
        "envira": "fa-envira",
        "erlang": "fa-erlang",
        "ethereum": "fa-ethereum",
        "etsy": "fa-etsy",
        "evernote": "fa-evernote",
        "expeditedssl": "fa-expeditedssl",
        "facebook": "fa-facebook",
        "facebookF": "fa-facebook-f",
        "facebookMessenger": "fa-facebook-messenger",
        "facebookSquare": "fa-facebook-square",
        "fantasyFlightGames": "fa-fantasy-flight-games",
        "fedex": "fa-fedex",
        "fedora": "fa-fedora",
        "figma": "fa-figma",
        "firefox": "fa-firefox",
        "firstOrder": "fa-first-order",
        "firstOrderAlt": "fa-first-order-alt",
        "firstdraft": "fa-firstdraft",
        "flickr": "fa-flickr",
        "flipboard": "fa-flipboard",
        "fly": "fa-fly",
        "fontAwesome": "fa-font-awesome",
        "fontAwesomeAlt": "fa-font-awesome-alt",
        "fontAwesomeFlag": "fa-font-awesome-flag",
        "fontAwesomeLogoFull": "fa-font-awesome-logo-full",
        "fonticons": "fa-fonticons",
        "fonticonsFi": "fa-fonticons-fi",
        "fortAwesome": "fa-fort-awesome",
        "fortAwesomeAlt": "fa-fort-awesome-alt",
        "forumbee": "fa-forumbee",
        "foursquare": "fa-foursquare",
        "freeCodeCamp": "fa-free-code-camp",
        "freebsd": "fa-freebsd",
        "fulcrum": "fa-fulcrum",
        "galacticRepublic": "fa-galactic-republic",
        "galacticSenate": "fa-galactic-senate",
        "getPocket": "fa-get-pocket",
        "gg": "fa-gg",
        "ggCircle": "fa-gg-circle",
        "git": "fa-git",
        "gitAlt": "fa-git-alt",
        "gitSquare": "fa-git-square",
        "github": "fa-github",
        "githubAlt": "fa-github-alt",
        "githubSquare": "fa-github-square",
        "gitkraken": "fa-gitkraken",
        "gitlab": "fa-gitlab",
        "gitter": "fa-gitter",
        "glide": "fa-glide",
        "glideG": "fa-glide-g",
        "gofore": "fa-gofore",
        "goodreads": "fa-goodreads",
        "goodreadsG": "fa-goodreads-g",
        "google": "fa-google",
        "googleDrive": "fa-google-drive",
        "googlePlay": "fa-google-play",
        "googlePlus": "fa-google-plus",
        "googlePlusG": "fa-google-plus-g",
        "googlePlusSquare": "fa-google-plus-square",
        "googleWallet": "fa-google-wallet",
        "gratipay": "fa-gratipay",
        "grav": "fa-grav",
        "gripfire": "fa-gripfire",
        "grunt": "fa-grunt",
        "gulp": "fa-gulp",
        "hackerNews": "fa-hacker-news",
        "hackerNewsSquare": "fa-hacker-news-square",
        "hackerrank": "fa-hackerrank",
        "hips": "fa-hips",
        "hireAHelper": "fa-hire-a-helper",
        "hooli": "fa-hooli",
        "hornbill": "fa-hornbill",
        "hotjar": "fa-hotjar",
        "houzz": "fa-houzz",
        "html5": "fa-html5",
        "hubspot": "fa-hubspot",
        "imdb": "fa-imdb",
        "instagram": "fa-instagram",
        "intercom": "fa-intercom",
        "internetExplorer": "fa-internet-explorer",
        "invision": "fa-invision",
        "ioxhost": "fa-ioxhost",
        "itchIo": "fa-itch-io",
        "itunes": "fa-itunes",
        "itunesNote": "fa-itunes-note",
        "java": "fa-java",
        "jediOrder": "fa-jedi-order",
        "jenkins": "fa-jenkins",
        "jira": "fa-jira",
        "joget": "fa-joget",
        "joomla": "fa-joomla",
        "js": "fa-js",
        "jsSquare": "fa-js-square",
        "jsfiddle": "fa-jsfiddle",
        "kaggle": "fa-kaggle",
        "keybase": "fa-keybase",
        "keycdn": "fa-keycdn",
        "kickstarter": "fa-kickstarter",
        "kickstarterK": "fa-kickstarter-k",
        "korvue": "fa-korvue",
        "laravel": "fa-laravel",
        "lastfm": "fa-lastfm",
        "lastfmSquare": "fa-lastfm-square",
        "leanpub": "fa-leanpub",
        "less": "fa-less",
        "line": "fa-line",
        "linkedin": "fa-linkedin",
        "linkedinIn": "fa-linkedin-in",
        "linode": "fa-linode",
        "linux": "fa-linux",
        "lyft": "fa-lyft",
        "magento": "fa-magento",
        "mailchimp": "fa-mailchimp",
        "mandalorian": "fa-mandalorian",
        "markdown": "fa-markdown",
        "mastodon": "fa-mastodon",
        "maxcdn": "fa-maxcdn",
        "medapps": "fa-medapps",
        "medium": "fa-medium",
        "mediumM": "fa-medium-m",
        "medrt": "fa-medrt",
        "meetup": "fa-meetup",
        "megaport": "fa-megaport",
        "mendeley": "fa-mendeley",
        "microsoft": "fa-microsoft",
        "mix": "fa-mix",
        "mixcloud": "fa-mixcloud",
        "mizuni": "fa-mizuni",
        "modx": "fa-modx",
        "monero": "fa-monero",
        "napster": "fa-napster",
        "neos": "fa-neos",
        "nimblr": "fa-nimblr",
        "node": "fa-node",
        "nodeJs": "fa-node-js",
        "npm": "fa-npm",
        "ns8": "fa-ns8",
        "nutritionix": "fa-nutritionix",
        "odnoklassniki": "fa-odnoklassniki",
        "odnoklassnikiSquare": "fa-odnoklassniki-square",
        "oldRepublic": "fa-old-republic",
        "opencart": "fa-opencart",
        "openid": "fa-openid",
        "opera": "fa-opera",
        "optinMonster": "fa-optin-monster",
        "osi": "fa-osi",
        "page4": "fa-page4",
        "pagelines": "fa-pagelines",
        "palfed": "fa-palfed",
        "patreon": "fa-patreon",
        "paypal": "fa-paypal",
        "pennyArcade": "fa-penny-arcade",
        "periscope": "fa-periscope",
        "phabricator": "fa-phabricator",
        "phoenixFramework": "fa-phoenix-framework",
        "phoenixSquadron": "fa-phoenix-squadron",
        "php": "fa-php",
        "piedPiper": "fa-pied-piper",
        "piedPiperAlt": "fa-pied-piper-alt",
        "piedPiperHat": "fa-pied-piper-hat",
        "piedPiperPp": "fa-pied-piper-pp",
        "pinterest": "fa-pinterest",
        "pinterestP": "fa-pinterest-p",
        "pinterestSquare": "fa-pinterest-square",
        "playstation": "fa-playstation",
        "productHunt": "fa-product-hunt",
        "pushed": "fa-pushed",
        "python": "fa-python",
        "qq": "fa-qq",
        "quinscape": "fa-quinscape",
        "quora": "fa-quora",
        "rProject": "fa-r-project",
        "raspberryPi": "fa-raspberry-pi",
        "ravelry": "fa-ravelry",
        "react": "fa-react",
        "reacteurope": "fa-reacteurope",
        "readme": "fa-readme",
        "rebel": "fa-rebel",
        "redRiver": "fa-red-river",
        "reddit": "fa-reddit",
        "redditAlien": "fa-reddit-alien",
        "redditSquare": "fa-reddit-square",
        "redhat": "fa-redhat",
        "renren": "fa-renren",
        "replyd": "fa-replyd",
        "researchgate": "fa-researchgate",
        "resolving": "fa-resolving",
        "rev": "fa-rev",
        "rocketchat": "fa-rocketchat",
        "rockrms": "fa-rockrms",
        "safari": "fa-safari",
        "salesforce": "fa-salesforce",
        "sass": "fa-sass",
        "schlix": "fa-schlix",
        "scribd": "fa-scribd",
        "searchengin": "fa-searchengin",
        "sellcast": "fa-sellcast",
        "sellsy": "fa-sellsy",
        "servicestack": "fa-servicestack",
        "shirtsinbulk": "fa-shirtsinbulk",
        "shopware": "fa-shopware",
        "simplybuilt": "fa-simplybuilt",
        "sistrix": "fa-sistrix",
        "sith": "fa-sith",
        "sketch": "fa-sketch",
        "skyatlas": "fa-skyatlas",
        "skype": "fa-skype",
        "slack": "fa-slack",
        "slackHash": "fa-slack-hash",
        "slideshare": "fa-slideshare",
        "snapchat": "fa-snapchat",
        "snapchatGhost": "fa-snapchat-ghost",
        "snapchatSquare": "fa-snapchat-square",
        "soundcloud": "fa-soundcloud",
        "sourcetree": "fa-sourcetree",
        "speakap": "fa-speakap",
        "speakerDeck": "fa-speaker-deck",
        "spotify": "fa-spotify",
        "squarespace": "fa-squarespace",
        "stackExchange": "fa-stack-exchange",
        "stackOverflow": "fa-stack-overflow",
        "stackpath": "fa-stackpath",
        "staylinked": "fa-staylinked",
        "steam": "fa-steam",
        "steamSquare": "fa-steam-square",
        "steamSymbol": "fa-steam-symbol",
        "stickerMule": "fa-sticker-mule",
        "strava": "fa-strava",
        "stripe": "fa-stripe",
        "stripeS": "fa-stripe-s",
        "studiovinari": "fa-studiovinari",
        "stumbleupon": "fa-stumbleupon",
        "stumbleuponCircle": "fa-stumbleupon-circle",
        "superpowers": "fa-superpowers",
        "supple": "fa-supple",
        "suse": "fa-suse",
        "swift": "fa-swift",
        "symfony": "fa-symfony",
        "teamspeak": "fa-teamspeak",
        "telegram": "fa-telegram",
        "telegramPlane": "fa-telegram-plane",
        "tencentWeibo": "fa-tencent-weibo",
        "theRedYeti": "fa-the-red-yeti",
        "themeco": "fa-themeco",
        "themeisle": "fa-themeisle",
        "thinkPeaks": "fa-think-peaks",
        "tradeFederation": "fa-trade-federation",
        "trello": "fa-trello",
        "tripadvisor": "fa-tripadvisor",
        "tumblr": "fa-tumblr",
        "tumblrSquare": "fa-tumblr-square",
        "twitch": "fa-twitch",
        "twitter": "fa-twitter",
        "twitterSquare": "fa-twitter-square",
        "typo3": "fa-typo3",
        "uber": "fa-uber",
        "ubuntu": "fa-ubuntu",
        "uikit": "fa-uikit",
        "uniregistry": "fa-uniregistry",
        "untappd": "fa-untappd",
        "ups": "fa-ups",
        "usb": "fa-usb",
        "usps": "fa-usps",
        "ussunnah": "fa-ussunnah",
        "vaadin": "fa-vaadin",
        "viacoin": "fa-viacoin",
        "viadeo": "fa-viadeo",
        "viadeoSquare": "fa-viadeo-square",
        "viber": "fa-viber",
        "vimeo": "fa-vimeo",
        "vimeoSquare": "fa-vimeo-square",
        "vimeoV": "fa-vimeo-v",
        "vine": "fa-vine",
        "vk": "fa-vk",
        "vnv": "fa-vnv",
        "vuejs": "fa-vuejs",
        "waze": "fa-waze",
        "weebly": "fa-weebly",
        "weibo": "fa-weibo",
        "weixin": "fa-weixin",
        "whatsapp": "fa-whatsapp",
        "whatsappSquare": "fa-whatsapp-square",
        "whmcs": "fa-whmcs",
        "wikipediaW": "fa-wikipedia-w",
        "windows": "fa-windows",
        "wix": "fa-wix",
        "wizardsOfTheCoast": "fa-wizards-of-the-coast",
        "wolfPackBattalion": "fa-wolf-pack-battalion",
        "wordpress": "fa-wordpress",
        "wordpressSimple": "fa-wordpress-simple",
        "wpbeginner": "fa-wpbeginner",
        "wpexplorer": "fa-wpexplorer",
        "wpforms": "fa-wpforms",
        "wpressr": "fa-wpressr",
        "xbox": "fa-xbox",
        "xing": "fa-xing",
        "xingSquare": "fa-xing-square",
        "yCombinator": "fa-y-combinator",
        "yahoo": "fa-yahoo",
        "yammer": "fa-yammer",
        "yandex": "fa-yandex",
        "yandexInternational": "fa-yandex-international",
        "yarn": "fa-yarn",
        "yelp": "fa-yelp",
        "yoast": "fa-yoast",
        "youtube": "fa-youtube",
        "youtubeSquare": "fa-youtube-square",
        "zhihu":  "fa-zhihu",
    ]
}
